// $Id: fe_bernstein_shape_0D.C 3391 2009-05-26 03:50:35Z benkirk $

// The libMesh Finite Element Library.
// Copyright (C) 2002-2008 Benjamin S. Kirk, John W. Peterson, Roy H. Stogner
  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
  
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
  
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


// C++ includes

// Local includes
#include "fe.h"
#include "elem.h"




template <>
Real FE<0,BERNSTEIN>::shape(const ElemType,
			   const Order,
			   const unsigned int i,
			   const Point&)
{
  libmesh_assert (i < 1);
  return 1.;
}



template <>
Real FE<0,BERNSTEIN>::shape(const Elem*,
			   const Order,
			   const unsigned int i,
			   const Point&)
{
  libmesh_assert (i < 1);
  return 1.;
}



template <>
Real FE<0,BERNSTEIN>::shape_deriv(const ElemType,
				 const Order,
				 const unsigned int,
				 const unsigned int,
				 const Point&)
{
  // No spatial derivatives in 0D!
  libmesh_error();
  return 0.;
}



template <>
Real FE<0,BERNSTEIN>::shape_deriv(const Elem*,
				 const Order,
				 const unsigned int,
				 const unsigned int,
				 const Point&)
{
  // No spatial derivatives in 0D!
  libmesh_error();
  return 0.;
}




template <>
Real FE<0,BERNSTEIN>::shape_second_deriv(const ElemType,
					const Order,
					const unsigned int,
					const unsigned int,
					const Point&)
{
  // No spatial derivatives in 0D!
  libmesh_error();
  return 0.;
}



template <>
Real FE<0,BERNSTEIN>::shape_second_deriv(const Elem*,
				        const Order,
				        const unsigned int,
				        const unsigned int,
				        const Point&)
{
  // No spatial derivatives in 0D!
  libmesh_error();
  return 0.;
}
